#include <GL/glut.h>
#include <GL/glu.h>
#include <GL/gl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include "sounddefs.h"
#include "defines.h"


extern int Lives;
extern int GameRunning;
extern int GameType;
extern int NumPlayers;
extern int BoomDelay;
extern int NextBoom;
extern int BoomPhase;
extern char PlayerName[MAX_PLAYERNAME_LENGTH];
extern int GameStop;
extern int myShip;
extern int ShowScore;
extern struct Model *Models[MAX_MODELS];
extern struct Model *AsteroidModels[MAX_OBJECTS];
extern struct Point ZeroPosition;
extern GLuint texture[6];
extern int ReverseView;
extern int volume;
extern int newvolume;
