#!/bin/bash

rm -rf ../atr3d
mkdir ../atr3d
cp atr3d.exe ../atr3d
cp -R data/conf ../atr3d
cp -R data/sounds ../atr3d
cp -R data/textures ../atr3d
cp -R data/models ../atr3d
mkdir ../atr3d/docs
cp GAMEPLAY ../atr3d/docs/GAMEPLAY.doc
cp README ../atr3d/docs/README.doc
cp COPYING ../atr3d/docs/COPYING.doc
cp /bin/cygwin1.dll ../atr3d
cp /bin/glut32.dll ../atr3d
