extern void KeyDown(unsigned char key, int x, int y);
extern void KeyUp(unsigned char key, int x, int y);
extern void MainMenuSpecialKey(int key, int x, int y);
extern void ConfMenuSpecialKey(int key, int x, int y);
extern void ScoreMenuSpecialKey(int key, int x, int y);
extern void MainMenuKey(unsigned char key, int x, int y);
extern void ConfMenuKey(unsigned char key, int x, int y);
extern void ScoreMenuKey(unsigned char key, int x, int y);
extern void SetKeyValueKey(unsigned char key, int x, int y);
extern void TextEntryKey(unsigned char key, int x, int y);
extern void ScoreEntryKey(unsigned char key, int x, int y);
extern void IntTextEntryKey(unsigned char key, int x, int y);


extern ControlStruct controls;
