extern char ServerConfigFileName[MAX_FILENAME_SIZE];
extern char ServerSingleConfigFileName[MAX_FILENAME_SIZE];


extern void ReadServerConfig(char *ConfigFileName);
extern void SetServerDefaults();
extern void *ServerAssignValue(char *VariableName, char *value);
extern void SaveServerConfig();
