/*
  Main class for any object that appears on the screen.  The ship is a derived
  class.
*/
class Object {

public:
  int Type;
  float Size;
  float Age;
  struct Point Position;
  float RotationMatrix[3][3];
  float CameraMatrix[3][3];
  struct Model *ObjModel;

  Object()
  {
  }
};

extern Object *Objects[MAX_OBJECTS];
