extern float MaxRockSize;
extern float MinRockSize;
extern float BotRatio;


extern void *StartGame(void *threadid);
extern void Accelerate(int playernum);
extern void DeletePlayer(int playernum);
extern void InitPlayerData(int playernum);
extern void CreateExplosion(int objnum, int type);
extern int BounceObjects(ServerObject *obj1, ServerObject *obj2);
extern void AddSound(int playernum, struct Point *SoundPos, char type);
extern void AddSoundAll(struct Point *SoundPos, char type);
extern float CalcDistance(struct Point *point1, struct Point *point2);
extern void CreateRandomPosition(struct Point *Position);
extern void CreateExplosionRandomRock(int objnum);
