/* Generic key control definitions */
#define TEST_PACKET    0x20

#define X_FORWARD_KEY  0x01
#define X_BACKWARD_KEY 0x02
#define Y_FORWARD_KEY  0x03
#define Y_BACKWARD_KEY 0x04
#define Z_FORWARD_KEY  0x05
#define Z_BACKWARD_KEY 0x06

#define X_FROTATE_KEY  0x07
#define X_BROTATE_KEY  0x08
#define Y_FROTATE_KEY  0x09
#define Y_BROTATE_KEY  0x0a
#define Z_FROTATE_KEY  0x0b
#define Z_BROTATE_KEY  0x0c

#define FIRE_KEY       0x0d
#define SHIELD_KEY     0x0e
#define ABORT_KEY      0x0f
#define NUKE_KEY       0x10
#define WARP_KEY       0x11
