class Ship:public Object {

public:
  int Score;
  int Lives;
  int ShieldOn;
  int Shield;
  int Armor;
  int Shaking;
  int Thrusting;
  char Alive;
  char Warps;
  char Nukes;
  char PlayerName[MAX_PLAYERNAME_LENGTH];

  Ship():Object()
  {
    Score=0;
    Lives=0;
    ShieldOn=0;
    Shield=0;
    Armor=0;
    Shaking=0;
    Thrusting=0;
    Alive=0;
    Warps=0;
    Nukes=0;
    strcpy(PlayerName, "UnNamedPlayer");
  }
};

extern Ship *Ships[MAX_PLAYERS];
