extern int GameSleep;
extern int MaxKills;
extern int ServerGameType;
extern int ShipDeadTime;
extern int StartLives;
extern int NumConnectedPlayers;
extern float ServerWorldWidth;
extern int ServerMaxShield;
extern int StartShield;
extern int ServerMaxArmor;
extern int StartArmor;
extern int PlayerConnected[MAX_PLAYERS];
extern int MaxPlayers;
extern int StopServer;
extern int NumAsteroids;
extern int StartAsteroids;
extern int NewAsteroidTime;
extern int AsteroidTimeSpeedup;
extern int MinAsteroidTime;
extern float DecelerationConstant;
extern float AccelerationConstant;
extern float ShipSize;
extern int ShotDamage;
extern int NukeDamage;
extern int NukeShieldDrain;
extern int CollisionDamage;
extern float ShotSpeed;
extern float NukeSpeed;
extern int PowerupAliveTime;
extern int NukeAliveTime;
extern int NukeExplosionAliveTime;
extern float NukeExplosionGrowthSpeed;
extern int ShotAliveTime;
extern float TotalAsteroidSize;
extern float MaxTotalAsteroidSize;
extern int ServerGameRunning;
extern int NumBots;
extern float BotRatio;
extern float StartBotRatio;
extern float MaxBotRatio;
extern float BotRatioIncrease;
extern int BotFireTime;
extern int BotNukeOdds;
extern int BotShieldTime;
extern int StartWarps;
extern int StartNukes;
extern int PrizeOdds;
extern int ShieldPowerupValue;
extern int ArmorPowerupValue;
extern float RotationSpeed;
