extern void InitMatrix(float Matrix[3][3]);
extern void MultMatrixes(float Matrix1[3][3], float Matrix2[3][3], int type);
extern void xRotateMatrix(float x, float Matrix[3][3], int type);
extern void yRotateMatrix(float y, float Matrix[3][3], int type);
extern void zRotateMatrix(float z, float Matrix[3][3], int type);
