extern void GenModelPoints(struct Model *model);
extern void AddModelDetail(struct Model **model);
extern void RandomizeModel(struct Model *model, float Randomization);
extern void RandomizeModelColors(struct Model *model);
extern void ShrinkModel(struct Model *model, float size);
extern void InvertModel(struct Model *model, int Type);
extern void ReCenterModel(struct Model *model);
extern float FindFurthestPoint(struct Model *model);
