#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <time.h>
#include <sys/time.h>


typedef struct {
  struct sockaddr_in PlayerIp;
  char Connected;
  struct timeval LastPacketTime;
} ConnectionStruct;


extern ConnectionStruct Connections[MAX_PLAYERS];
extern int listenfd;
extern int PacketSize;
extern int NetSleepTime;
extern int GamePort;
