#define NUM_HIGHSCORES          10


typedef struct {
  int Score;
  char Name[MAX_PLAYERNAME_LENGTH];
} HighScore;


extern HighScore HighScores[NUM_HIGHSCORES];
extern char HighScoreName[MAX_PLAYERNAME_LENGTH];
extern int Score;
extern char ScoreFileName[MAX_FILENAME_SIZE];


extern void ClearHighScores();
extern void InitHighScores();
extern int CheckForHighScore();
extern void AddHighScore(char *name);
extern void ReadHighScores();
extern void SaveHighScores();
