// Maximum number of sounds that can play at once
#define MAX_SOUNDS 32
#define MAX_LOOPING_SOUNDS 16

// Sound resource definitions...
#define ShotSound         0
#define HighScoreSound    1
#define ExplosionSound    2
#define ShipHitSound      3
#define Boom1             4
#define Boom2             5
#define ErrorSound        6
#define LuckySound        7
#define PingSound         8
#define BonkSound         9
#define ShieldOnSound    10
#define NoShieldSound    11
#define PrizeSound       12
#define CrunchSound      13
#define NewLifeSound     14
#define PrizeAppears     15
#define ThrustSound      16
#define IdiotSound       17
#define WarpSound        18
#define FireNukeSound    19
#define NukeExplodeSound 20
#define NukeAlertSound   21
#define NUM_SOUNDS       22
