extern int (*ObjectToObjectCollisionFunctions[MAX_TYPES][MAX_TYPES])(int, int);
extern int (*ObjectToShipCollisionFunctions[MAX_TYPES])(int, int);


extern int ShipToShipCollision(int ship1, int ship2);
extern void InitCollisionFunctions();
