extern int WarpSoundNum;
extern int ThrustSoundNum;
extern int ShieldSoundNum;


extern void CleanupSounds();
extern void setvolume();
extern void playsound(int soundnum, struct Point *Position=&ZeroPosition);
extern int playsoundinloop(int soundnum);
extern void stopsoundinloop(int soundnum);
extern void initsound(int *argc, char *argv[]);
